package com.iamberry.wechat.handles.agentInfo;

import com.iamberry.wechat.core.entity.PageBean;
import com.iamberry.wechat.core.entity.ResultMsg;
import com.iamberry.wechat.core.entity.WechatUtils;
import com.iamberry.wechat.core.entity.agentInfo.AgentConfig;
import com.iamberry.wechat.core.entity.agentInfo.AgentOrder;
import com.iamberry.wechat.core.entity.agentInfo.AgentOrderItem;
import com.iamberry.wechat.core.entity.agentInfo.AgentTooth;
import com.iamberry.wechat.core.entity.cart.SendPayDto;
import com.iamberry.wechat.core.entity.member.Member;
import com.iamberry.wechat.core.entity.pay.PayResult;
import com.iamberry.wechat.core.entity.product.Product;
import com.iamberry.wechat.face.agentInfo.AgentInfoService;
import com.iamberry.wechat.face.agentInfo.AgentPurchaseOrderService;
import com.iamberry.wechat.face.cart.ProductInfoService;
import com.iamberry.wechat.face.pay.PayService;
import com.iamberry.wechat.tools.NameUtils;
import com.iamberry.wechat.tools.OrderNOUtil;
import com.iamberry.wechat.tools.ResultInfo;
import com.iamberry.wechat.utils.Result;
import net.sf.json.JSONArray;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 采购&订单
 */
@Controller
@RequestMapping("/wechat/agentOrder")
public class AgentOrderHandler {
    @Autowired
    private AgentInfoService agentInfoService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private AgentPurchaseOrderService agentPurchaseOrderService;
    @Autowired
    private PayService payService;

    /**
     * 获取采购信息
     *
     * @return
     */
    @ResponseBody
    @RequestMapping("/getProduce")
    public ResultMsg getProduce(HttpServletRequest request) {
        ResultMsg rm = new ResultMsg();
        //Member member =  WechatUtils.getUserBySession(request);
        //String agentOpenid = member.getUserOpenid();

        String agentOpenid = "o-icasz5a10CfmJ_s-hvW-Ltqwtc";

        AgentTooth agentTooth = new AgentTooth();
        agentTooth.setAgentOpenid(agentOpenid);
        agentTooth = agentInfoService.getAgentTooth(agentTooth);
        if (agentTooth == null) {
            rm.setStatus(false);
            rm.setMessage("未找到该用户!");
            rm.setResultCode(ResultInfo.ERRORCODE);
            return rm;
        }

        Product product = new Product();
        List<Product> productList = productInfoService.listSelectProduct(product);

        for (Product pd : productList) {
            AgentConfig agentConfig = new AgentConfig();
            agentConfig.setColorId(pd.getColorId());
            agentConfig.setAgentId(agentTooth.getAgentId());
            agentConfig = agentInfoService.getAgentConfig(agentConfig);
            if (agentConfig != null && agentConfig.getConfigAmount() != null && agentConfig.getConfigAmount() != 0) {
                pd.setConfigAmount(agentConfig.getConfigAmount());
            } else {
                pd.setConfigAmount(pd.getProductDiscount());
            }
        }
        rm.setStatus(true);
        rm.setMessage("查询成功!");
        rm.setResultCode("200");
        rm.setData(productList);
        return rm;
    }

    /**
     * 去支付，调用方法
     *
     * @param request
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(value = "/payBefore", method = RequestMethod.POST)
    public ResultMsg payBefore(HttpServletRequest request, AgentOrder agentOrder) throws Exception {
        ResultMsg msg = new ResultMsg();
        msg.setResultCode(ResultInfo.ERRORCODE);
        msg.setStatus(false);

        //获取代理商信息
//        Member member = WechatUtils.getUserBySession(request);
//        /** 数据格式校验 */
//        if (member == null || member.getUserOpenid() == null) {// 校验当前用户信息是否丢失
//            msg.setMessage(ResultInfo.loginOutError);
//            return msg;
//        }
//        String openId = member.getUserOpenid();

        String openId = "o-icasz5a10CfmJ_s-hvW-Ltqwtc";
        Integer codeId = 101;

        AgentTooth agentTooth = new AgentTooth();
        agentTooth.setAgentOpenid(openId);
        agentTooth = agentInfoService.getAgentTooth(agentTooth);
        if (agentTooth == null) {
            msg.setMessage("未找到该用户!");
            return msg;
        }

        //验证前台传过来的数据是否规范
        if (!isValiAgentOrder(agentOrder)) {
            msg.setMessage("订单格式内容出错!");
            return msg;
        }

        //---------------生成订单 --- start ---------------------
        JSONArray jsonArray = JSONArray.fromObject(agentOrder.getAgentOrderItemJson());
        List<AgentOrderItem> agentOrderItemList = (List) JSONArray.toCollection(jsonArray, AgentOrderItem.class);
        if (agentOrderItemList == null || agentOrderItemList.size() < 1) {
            msg.setMessage("未选择产品!");
            return msg;
        }
        agentOrder.setAgentOrderid(OrderNOUtil.createOrderCode(codeId));
        agentOrder.setAgentId(agentTooth.getAgentId());
        Map<String, Object> map = agentPurchaseOrderService.addAgentOrder(agentOrder, agentOrderItemList);
        if (map.get("status") == null || map.get("status") == 400) {
            msg.setMessage(map.get("msg").toString());
            return msg;
        }
        //---------------生成订单 --- end -----------------------

        //支付
        ResultMsg resultMsg = this.pay((AgentOrder) map.get("agentOrder"), openId);

        if (resultMsg.getResultCode() == ResultInfo.SUCCESSCODE) {
            msg.setResultCode(ResultInfo.SUCCESSCODE);
            msg.setMessage(NameUtils.getConfig("SUCCESSINFO"));
            msg.setStatus(true);
        } else {
            msg.setResultCode(ResultInfo.ERRORCODE);
            msg.setMessage(NameUtils.getConfig("ERRORINFO"));
            msg.setStatus(true);
        }
        msg.setData(resultMsg.getData());
        return msg;
    }

    /**
     * 代理商订单支付调用
     *
     * @param agentOrder 订单
     * @param openId
     * @return
     */
    private ResultMsg pay(AgentOrder agentOrder, String openId) {
        boolean flag = false;
        ResultMsg msg = new ResultMsg();

        SendPayDto dto = new SendPayDto();
        dto.setSuccess(false);
        //查询订单项
        List<AgentOrderItem> agentOrderList = agentPurchaseOrderService.listAgentOrderItemByOrderId(agentOrder.getAgentOrderid());
        if (agentOrderList == null || agentOrderList.size() < 1) {
            msg.setResultCode(ResultInfo.ERRORCODE);
            msg.setMessage("查询订单报错");
            msg.setStatus(false);
            msg.setData(dto);
            return msg;
        }

        if (agentOrder.getAgentYetAmount() > 0) {
            // 发起支付
            PayResult result = payService.requestPay(openId, agentOrder.getAgentOrderid(), agentOrder.getAgentYetAmount(), agentOrderList.get(0).getItemProductName() + "...", NameUtils.getConfig("WECHAT_AGENT_PURCHASE_ORDER_BACK"));
            if (result.isPaySuccess()) {
                dto.setInfo(result.getPayData());
                dto.setSuccess(true);
                dto.setOrderId(agentOrder.getAgentOrderid());
                flag = true;
            } else {
                dto.setSuccess(false);
                dto.setInfo(result.getPayData());
                dto.setOrderId(agentOrder.getAgentOrderid());
                flag = false;
            }
        } else if (agentOrder.getAgentYetAmount() == 0) {
            dto.setInfo("isOK");
            dto.setSuccess(true);
            dto.setOrderId(agentOrder.getAgentOrderid());
            flag = true;
        } else {
            msg.setResultCode(ResultInfo.ERRORCODE);
            msg.setMessage(ResultInfo.paramFormatError);
            return msg;
        }

        if (flag) {
            msg.setResultCode(ResultInfo.SUCCESSCODE);
            msg.setMessage(NameUtils.getConfig("SUCCESSINFO"));
            msg.setData(dto);
            msg.setStatus(true);
        } else {
            msg.setResultCode(ResultInfo.ERRORCODE);
            msg.setMessage(NameUtils.getConfig("ERRORINFO"));
            msg.setData(dto);
            msg.setStatus(false);
        }
        return msg;
    }

    /**
     * 支付前的验证
     *
     * @param agentOrder
     * @return
     */
    public boolean isValiAgentOrder(AgentOrder agentOrder) {
        if (agentOrder.getAgentAddressInfo() == null || agentOrder.getAgentAddressInfo() == "") {
            return false;
        }
        if (agentOrder.getAgentAddressName() == null || agentOrder.getAgentAddressName() == "") {
            return false;
        }
        if (agentOrder.getAgentAddressPostnum() == null || agentOrder.getAgentAddressPostnum() == "") {
            return false;
        }
        if (agentOrder.getAgentAddressTel() == null) {
            return false;
        }
        if (agentOrder.getAgentOrderItemJson() == null || agentOrder.getAgentOrderItemJson() == "") {
            return false;
        }
        return true;
    }

    /**
     * 分页查询代理商采购订单
     * @param request
     * @param pageSize
     * @param pageNO
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping("/select_agent_order_list")
    public ResultMsg selectAgentOrderList(HttpServletRequest request,
                                          @RequestParam(value= "pageSize",defaultValue= "6" ,required=false) Integer pageSize,
                                          @RequestParam(value = "pageNO", defaultValue = "1") Integer pageNO) throws Exception {
        ResultMsg msg = new ResultMsg();
        Map<String,Object> map = new HashMap<String,Object>();
        Member member = WechatUtils.getUserBySession(request);
        AgentTooth agent = new AgentTooth();
        AgentOrder agentOrder = new AgentOrder();
        String status = request.getParameter("status");
        if (StringUtils.isNotEmpty(status)) {
            agentOrder.setAgentStatus(Integer.parseInt(status));
        }
        agent.setAgentOpenid(member.getUserOpenid());
        //获取代理商信息
        agent = agentInfoService.getAgentTooth(agent);
        if (agent == null) {
            msg.setResultCode(ResultInfo.ERRORCODE);
            msg.setStatus(false);
            msg.setMessage("您不是代理商,没有采购订单!");
            return msg;
        }

        agentOrder.setAgentId(agent.getAgentId());
        PageBean page = new PageBean();
        page.setPageNumber(pageNO);
        page.setPageSize(pageSize);
        page.initRecordBegin();
        agentOrder.setPage(page);
        //获取订单集合
        List<AgentOrder> agentOrderList = agentPurchaseOrderService.listAgentOrder(agentOrder);
        map.put("agent",agent);
        map.put("agentOrderList",agentOrderList);
        msg.setData(map);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        return msg;
    }
}
